import path from "path";
import CppProject from "../Project/CppProject.js";
import { getGlobalProjects, getProjectsByTypeName } from "../Tools/Projects.js";
import Builder from "./Builder.js";
import { getBuildDir } from "../Tools/BuildDir.js";
import fs from "fs"
import printDebug from "../Tools/DebugPrint.js";
import sourceFilesToString from "../Tools/SourceFilesToString.js";
import execAsync from "../Tools/ExecAsync.js";
import protectPath from "../Tools/ProtectPath.js";
import printErrorOrDebug from "../Tools/PrintErrorOrDebug.js";
import getExecutableExtension from "../Tools/GetExecutableExtension.js";

export default class CppBuilder implements Builder{
    compile = async ()=>{
        printDebug("compiling")
        const cppProjects  = getProjectsByTypeName("cpp") as CppProject[]
        const tasks:Promise<{
            stdout: string;
            stderr: string;
        }>[] = []
        cppProjects.forEach((e)=>{
            e.sourceFilesPath.forEach((v)=>{
                const objDir = path.join(getBuildDir(),e.name,".objs")
                printDebug(`making dir ${objDir}`)
                if(!fs.existsSync(objDir))fs.mkdirSync(objDir,{recursive:true});
                const objPath = path.join(objDir,path.parse(v).name+".o")
                e.objs.push(objPath)
                const command  = `${protectPath(e.compiler.compilerPath)} -c ${protectPath(v)} -o ${protectPath(objPath)} ${e.optimizeToBuildString()}`
                tasks.push(execAsync(command))
                printDebug(command)
            })
        });
        try{
            (await Promise.all(tasks)).forEach(e=>{
                printDebug(`stdout:${e.stdout}`)
                if(e.stderr){printErrorOrDebug(`stderr:${e.stderr}`)}
            })
        }catch(e:any){
            printErrorOrDebug(e.message)
        }
    }
    link = async()=>{
        printDebug("linking")
        const cppProjects  = getProjectsByTypeName("cpp") as CppProject[]
        const tasks:Promise<{
            stdout: string;
            stderr: string;
        }>[] =[]
        cppProjects.forEach(i=>{
            const command = `${protectPath(i.compiler.compilerPath)} ${sourceFilesToString(i.objs)} -o ${path.join(getBuildDir(),i.name,`${i.name}${getExecutableExtension()}`)}`
            printDebug(command)
            tasks.push(execAsync(command))
        })
    }
    build = async () =>{
        printDebug("building")
        await this.compile()
        await this.link()
    }
}